/*

Copyright (C) 2008 Louai Al-Khanji <louai.khanji@gmail.com>

This software is provided 'as-is', without any express or implied
warranty.  In no event will the authors be held liable for any damages
arising from the use of this software.

Permission is granted to anyone to use this software for any purpose,
including commercial applications, and to alter it and redistribute it
freely, subject to the following restrictions:

1. The origin of this software must not be misrepresented; you must not
claim that you wrote the original software. If you use this software
in a product, an acknowledgment in the product documentation would be
appreciated but is not required.
2. Altered source versions must be plainly marked as such, and must not be
misrepresented as being the original software.
3. This notice may not be removed or altered from any source distribution.

*/

#include <QLabel>
#include <QHBoxLayout>
#include <QFileInfo>
#include <QApplication>

#include <mediaobject.h>
#include <mediasource.h>

#include "nowplayinglabel.h"

NowPlayingLabel::NowPlayingLabel(Phonon::MediaObject* mo, QWidget* parent)
: QLabel(parent)
, m_mediaObject(mo)
{
    setText(tr("Welcome to %1").arg(qApp->applicationName()));

    connect(mo, SIGNAL(currentSourceChanged(Phonon::MediaSource)),
             this, SLOT(currentSourceChanged(Phonon::MediaSource)));
    connect(mo, SIGNAL(metaDataChanged()),
             this, SLOT(metaDataChanged()));
    connect(mo, SIGNAL(finished()),
             this, SLOT(finished()));
}

void NowPlayingLabel::setText(const QString& text)
{
    QLabel::setText(text);
    emit textChanged(this->text());
}

void NowPlayingLabel::currentSourceChanged(const Phonon::MediaSource& source)
{
    setText(QFileInfo(source.fileName()).fileName());
}

void NowPlayingLabel::metaDataChanged()
{
    QString artist = m_mediaObject->metaData(Phonon::ArtistMetaData)[0];
    QString title = m_mediaObject->metaData(Phonon::TitleMetaData)[0];
    if (!artist.isEmpty() || !title.isEmpty()) {
        QString s = QString("%1 - %2").arg(artist).arg(title);
        setText(s);
    }
}

void NowPlayingLabel::finished()
{
    setText(tr("Playlist finished"));
}
